@@ -16,11 +16,13 @@
    Boston, MA 02110-1301, USA.
 */
 
+#include "kateargumenthintmodel.h"
+
 #include <QTextFormat>
 #include <QGridLayout>
+#include <kapplication.h>
 
 #include <ktexteditor/codecompletionmodel.h>
-#include "kateargumenthintmodel.h"
 #include "katecompletionwidget.h"
 #include "kateargumenthinttree.h"
 #include "katecompletiontree.h"
@@ -42,7 +44,7 @@ void KateArgumentHintModel::buildRows() {
   QMap<int, QList<int> > m_depths; //Map each hint-depth to a list of functions of that depth
   for( int a = 0; a < group()->rows.count(); a++ ) {
     KateCompletionModel::ModelRow source = group()->rows[a];
-    QModelIndex  sourceIndex = source.first->index(source.second, 0);
+    QModelIndex  sourceIndex = source.second.sibling(source.second.row(), 0);
     QVariant v = sourceIndex.data(CodeCompletionModel::ArgumentHintDepth);
     if( v.type() == QVariant::Int ) {
       QList<int>& lst( m_depths[v.toInt()] );
@@ -67,7 +69,7 @@ KateArgumentHintModel::KateArgumentHintModel( KateCompletionWidget* parent ) : E
 
 QVariant KateArgumentHintModel::data ( const QModelIndex & index, int role ) const {
   if( index.row() <  0 || index.row() >= m_rows.count() ) {
-    //kDebug() << "KateArgumentHintModel::data: index out of bound: " << index.row() << " total rows: " << m_rows.count();
+    //kDebug( 13035 ) << "KateArgumentHintModel::data: index out of bound: " << index.row() << " total rows: " << m_rows.count();
     return QVariant();
   }
 
@@ -76,22 +78,22 @@ QVariant KateArgumentHintModel::data ( const QModelIndex & index, int role ) con
     if( role == Qt::DisplayRole && index.column() == 0 ) {
       return QString("Depth %1").arg(-m_rows[index.row()]);
     } else if( role == Qt::BackgroundRole ) {
-      return QColor(Qt::black);
+      return KApplication::kApplication()->palette().toolTipBase().color();
     }else if( role == Qt::ForegroundRole ) {
-      return QColor(Qt::white);
+      return KApplication::kApplication()->palette().toolTipText().color();
     }else{
       return QVariant();
     }
   }
 
   if( m_rows[index.row()] <  0 || m_rows[index.row()] >= group()->rows.count() ) {
-    kDebug() << "KateArgumentHintModel::data: index out of bound: " << m_rows[index.row()] << " total rows: " << group()->rows.count();
+    kDebug( 13035 ) << "KateArgumentHintModel::data: index out of bound: " << m_rows[index.row()] << " total rows: " << group()->rows.count();
     return QVariant();
   }
   
   KateCompletionModel::ModelRow source = group()->rows[m_rows[index.row()]];
   if( !source.first ) {
-    kDebug() << "KateArgumentHintModel::data: Row does not exist in source";
+    kDebug( 13035 ) << "KateArgumentHintModel::data: Row does not exist in source";
     return QVariant();
   }
 
@@ -113,10 +115,10 @@ QVariant KateArgumentHintModel::data ( const QModelIndex & index, int role ) con
     }
   }
 
-   QModelIndex  sourceIndex = source.first->index(source.second, index.column());
+  QModelIndex  sourceIndex = source.second.sibling(source.second.row(), index.column());
  
   if( !sourceIndex.isValid() ) {
-    kDebug() << "KateArgumentHintModel::data: Source-index is not valid";
+    kDebug( 13035 ) << "KateArgumentHintModel::data: Source-index is not valid";
     return QVariant();
   }
 
@@ -127,7 +129,7 @@ QVariant KateArgumentHintModel::data ( const QModelIndex & index, int role ) con
       QString totalText;
       for( int a = CodeCompletionModel::Prefix; a <= CodeCompletionModel::Postfix; a++ )
         if( a != CodeCompletionModel::Scope ) //Skip the scope
-          totalText += source.first->index(source.second, a).data(Qt::DisplayRole).toString() + ' ';
+          totalText += source.second.sibling(source.second.row(), a).data(Qt::DisplayRole).toString() + ' ';
     
       
       return QVariant(totalText);
@@ -135,10 +137,11 @@ QVariant KateArgumentHintModel::data ( const QModelIndex & index, int role ) con
     case CodeCompletionModel::HighlightingMethod:
     {
       //Return that we are doing custom-highlighting of one of the sub-strings does it
-      for( int a = CodeCompletionModel::Prefix; a <= CodeCompletionModel::Postfix; a++ )
-          if( source.first->index(source.second, a).data(CodeCompletionModel::HighlightingMethod).type() == QVariant::Int ) {
+      for( int a = CodeCompletionModel::Prefix; a <= CodeCompletionModel::Postfix; a++ ) {
+          QVariant method = source.second.sibling(source.second.row(), a).data(CodeCompletionModel::HighlightingMethod);
+          if( method.type() == QVariant::Int && method.toInt() ==  CodeCompletionModel::CustomHighlighting)
             return QVariant(CodeCompletionModel::CustomHighlighting);
-          }
+      }
     
       return QVariant();
     }
@@ -148,13 +151,13 @@ QVariant KateArgumentHintModel::data ( const QModelIndex & index, int role ) con
       
       //Collect strings
       for( int a = CodeCompletionModel::Prefix; a <= CodeCompletionModel::Postfix; a++ )
-          strings << source.first->index(source.second, a).data(Qt::DisplayRole).toString();
+          strings << source.second.sibling(source.second.row(), a).data(Qt::DisplayRole).toString();
 
       QList<QVariantList> highlights;
 
       //Collect custom-highlightings
       for( int a = CodeCompletionModel::Prefix; a <= CodeCompletionModel::Postfix; a++ )
-          highlights << source.first->index(source.second, a).data(CodeCompletionModel::CustomHighlight).toList();
+          highlights << source.second.sibling(source.second.row(), a).data(CodeCompletionModel::CustomHighlight).toList();
 
       //Replace invalid QTextFormats with match-quality color or yellow.
       for( QList<QVariantList>::iterator it = highlights.begin(); it != highlights.end(); ++it )
@@ -184,12 +187,12 @@ QVariant KateArgumentHintModel::data ( const QModelIndex & index, int role ) con
       }
       
       
-      return mergeCustomHighlighting( strings, highlights );
+      return mergeCustomHighlighting( strings, highlights, 1 );
     }
     case Qt::DecorationRole:
     {
       //Redirect the decoration to the decoration of the item-column
-      return source.first->index(source.second, CodeCompletionModel::Icon).data(role);
+      return source.second.sibling(source.second.row(), CodeCompletionModel::Icon).data(role);
     }
   }
   
@@ -243,7 +246,7 @@ int KateArgumentHintModel::contextMatchQuality(const QModelIndex& index) const {
   if( !source.first )
     return -1;
   
-   QModelIndex  sourceIndex = source.first->index(source.second, 0);
+   QModelIndex  sourceIndex = source.second.sibling(source.second.row(), 0);
  
   if( !sourceIndex.isValid() )
     return -1;
