@@ -88,6 +88,8 @@
 #include <QtCore/QTextStream>
 #include <QtCore/QMimeData>
 #include <QtCore/QTextCodec>
+#include <QtCore/QMutexLocker>
+
 //END includes
 
 static void blockFix(KTextEditor::Range& range)
@@ -103,6 +105,7 @@ static void blockFix(KTextEditor::Range& range)
 KateView::KateView( KateDocument *doc, QWidget *parent )
     : KTextEditor::View( parent )
     , m_completionWidget(0)
+    , m_annotationModel(0)
     , m_editActions (0)
     , m_doc( doc )
     , m_spell( new KateSpell( this ) )
@@ -118,6 +121,8 @@ KateView::KateView( KateDocument *doc, QWidget *parent )
     , m_searchBar (0)
     , m_gotoBar (0)
 {
+  setComponentData ( KateGlobal::self()->componentData () );
+
   KateGlobal::self()->registerView( this );
 
   m_config = new KateViewConfig (this);
@@ -126,6 +131,12 @@ KateView::KateView( KateDocument *doc, QWidget *parent )
 
   m_viewInternal = new KateViewInternal( this, doc );
 
+  // ugly workaround:
+  // Force the layout to be left-to-right even on RTL deskstop, as discussed
+  // on the mailing list. This will cause the lines and icons panel to be on
+  // the left, even for Arabic/Hebrew/Farsi/whatever users.
+  setLayoutDirection ( Qt::LeftToRight );
+
   // layouting ;)
   m_vBox = new QVBoxLayout (this);
   m_vBox->setMargin (0);
@@ -172,20 +183,19 @@ KateView::KateView( KateDocument *doc, QWidget *parent )
   // this really is needed :)
   m_viewInternal->updateView ();
 
-  setComponentData( KateGlobal::self()->componentData() );
   doc->addView( this );
 
   setFocusProxy( m_viewInternal );
   setFocusPolicy( Qt::StrongFocus );
 
-  if (!doc->singleViewMode()) {
-    setXMLFile( "katepartui.rc" );
-  } else {
-    if( doc->readOnly() )
-      setXMLFile( "katepartreadonlyui.rc" );
-    else
-      setXMLFile( "katepartui.rc" );
-  }
+  // default ui file with all features
+  QString uifile = "katepartui.rc";
+
+  // simple mode
+  if (doc->simpleMode ())
+    uifile = "katepartsimpleui.rc";
+
+  setXMLFile( uifile );
 
   setupConnections();
   setupActions();
@@ -202,6 +212,7 @@ KateView::KateView( KateDocument *doc, QWidget *parent )
   updateConfig ();
 
   slotHlChanged();
+  KCursor::setAutoHideCursor( m_viewInternal, true );
   /*test texthint
   connect(this,SIGNAL(needTextHint(int, int, QString &)),
   this,SLOT(slotNeedTextHint(int, int, QString &)));
@@ -246,6 +257,9 @@ void KateView::setupConnections()
   connect( m_viewInternal, SIGNAL(dropEventPass(QDropEvent*)),
            this,           SIGNAL(dropEventPass(QDropEvent*)) );
 
+  connect( m_doc, SIGNAL(annotationModelChanged( KTextEditor::AnnotationModel*, KTextEditor::AnnotationModel* )),
+           m_viewInternal->m_leftBorder, SLOT(annotationModelChanged( KTextEditor::AnnotationModel*, KTextEditor::AnnotationModel* )) );
+
   if ( m_doc->browserView() )
   {
     connect( this, SIGNAL(dropEventPass(QDropEvent*)),
@@ -311,7 +325,7 @@ void KateView::setupActions()
 
     a = ac->addAction("tools_cleanIndent");
     a->setText(i18n("&Clean Indentation"));
-    a->setWhatsThis(i18n("Use this to clean the indentation of a selected block of text (only tabs/only spaces)<br /><br />"
+    a->setWhatsThis(i18n("Use this to clean the indentation of a selected block of text (only tabs/only spaces).<br /><br />"
         "You can configure whether tabs should be honored and used or replaced with spaces, in the configuration dialog."));
     connect(a, SIGNAL(triggered(bool)), SLOT(cleanIndent()));
 
@@ -337,6 +351,7 @@ void KateView::setupActions()
 
     a = m_toggleWriteLock = new KToggleAction(i18n("&Read Only Mode"), this);
     a->setWhatsThis( i18n("Lock/unlock the document for writing") );
+    a->setChecked( !m_doc->isReadWrite() );
     connect(a, SIGNAL(triggered(bool)), SLOT( toggleWriteLock() ));
     ac->addAction("tools_toggle_write_lock", a);
 
@@ -403,7 +418,7 @@ void KateView::setupActions()
 
   KateModeMenu *ftm = new KateModeMenu (i18n("&Mode"), this);
   ac->addAction("tools_mode", ftm);
-  ftm->setWhatsThis(i18n("Here you can choose which mode should be used for the current document. This will influence the used highlighting and folding for example."));
+  ftm->setWhatsThis(i18n("Here you can choose which mode should be used for the current document. This will influence the highlighting and folding being used, for example."));
   ftm->updateMenu (m_doc);
 
   KateHighlightingMenu *menu = new KateHighlightingMenu (i18n("&Highlighting"), this);
@@ -495,7 +510,7 @@ void KateView::setupActions()
 
   a = m_toggleScrollBarMarks = toggleAction = new KToggleAction(i18n("Show Scroll&bar Marks"), this);
   ac->addAction("view_scrollbar_marks", a);
-  a->setWhatsThis(i18n("Show/hide the marks on the vertical scrollbar.<br /><br />The marks, for instance, show bookmarks."));
+  a->setWhatsThis(i18n("Show/hide the marks on the vertical scrollbar.<br /><br />The marks show bookmarks, for instance."));
   connect(a, SIGNAL(triggered(bool)), SLOT(toggleScrollBarMarks()));
 
   a = toggleAction = m_toggleWWMarker = new KToggleAction(i18n("Show Static &Word Wrap Marker"), this);
@@ -554,17 +569,15 @@ void KateView::setupActions()
   a->setWhatsThis(i18n("Look up a piece of text or regular expression and replace the result with some given text."));
 
   m_spell->createActions( ac );
-  m_bookmarks->createActions( ac );
+
+  if (!m_doc->simpleMode ())
+    m_bookmarks->createActions( ac );
 
   slotSelectionChanged ();
 
   ac->addAssociatedWidget(m_viewInternal);
   foreach (QAction* action, ac->actions())
-#if QT_VERSION < KDE_MAKE_VERSION(4,4,0)
-    action->setShortcutContext(Qt::WidgetShortcut); // remove after Qt4.4 becomes mandatory
-#else
     action->setShortcutContext(Qt::WidgetWithChildrenShortcut);
-#endif
 
   connect (this, SIGNAL(selectionChanged(KTextEditor::View*)), this, SLOT(slotSelectionChanged()));
 }
@@ -809,12 +822,9 @@ void KateView::setupEditActions()
     slotLostFocus();
 
   m_editActions->addAssociatedWidget(m_viewInternal);
+
   foreach (QAction* action, m_editActions->actions())
-#if QT_VERSION < KDE_MAKE_VERSION(4,4,0)
-    action->setShortcutContext(Qt::WidgetShortcut); // remove after Qt4.4 becomes mandatory
-#else
     action->setShortcutContext(Qt::WidgetWithChildrenShortcut);
-#endif
 }
 
 void KateView::setupCodeFolding()
@@ -968,6 +978,8 @@ void KateView::contextMenuEvent( QContextMenuEvent *ev )
 
 bool KateView::setCursorPositionInternal( const KTextEditor::Cursor& position, uint tabwidth, bool calledExternally )
 {
+  QMutexLocker lock(m_doc->smartMutex());
+
   KateTextLine::Ptr l = m_doc->kateTextLine( position.line() );
 
   if (!l)
@@ -1263,6 +1275,9 @@ void KateView::updateDocumentConfig()
 
   m_updatingDocumentConfig = false;
 
+  // maybe block selection or wrap-cursor mode changed
+  ensureCursorColumnValid();
+
   m_viewInternal->updateView (true);
 
   m_renderer->setTabWidth (m_doc->config()->tabWidth());
@@ -1277,6 +1292,7 @@ void KateView::updateRendererConfig()
   m_toggleWWMarker->setChecked( m_renderer->config()->wordWrapMarker()  );
 
   m_viewInternal->updateBracketMarkAttributes();
+  m_viewInternal->updateBracketMarks();
 
   // update the text area
   m_viewInternal->updateView (true);
@@ -1309,6 +1325,21 @@ void KateView::updateFoldingConfig ()
       a->setEnabled (m_doc->highlight() && m_doc->highlight()->allowsFolding());
 }
 
+void KateView::ensureCursorColumnValid()
+{
+  KTextEditor::Cursor c = m_viewInternal->getCursor();
+
+  // make sure the cursor is valid:
+  // - in block selection mode or if wrap cursor is off, the colum is arbitrary
+  // - otherwise: it's bounded by the line length
+  if (!blockSelectionMode() && wrapCursor()
+      && (!c.isValid() || c.column() > m_doc->lineLength(c.line())))
+  {
+    c.setColumn(m_doc->kateTextLine(cursorPosition().line())->length());
+    setCursorPosition(c);
+  }
+}
+
 //BEGIN EDIT STUFF
 void KateView::editStart ()
 {
@@ -1412,6 +1443,8 @@ void KateView::notifyMousePositionChanged(const KTextEditor::Cursor& newPosition
 
 bool KateView::setSelection( const KTextEditor::Range &selection )
 {
+  QMutexLocker l(m_doc->smartMutex());
+
   KTextEditor::Range oldSelection = *m_selection;
   *m_selection = selection;
 
@@ -1431,6 +1464,8 @@ bool KateView::clearSelection()
 
 bool KateView::clearSelection(bool redraw, bool finishedChangingSelection)
 {
+  QMutexLocker l(m_doc->smartMutex());
+
   if( !selection() )
     return false;
 
@@ -1461,6 +1496,8 @@ bool KateView::selection() const
 
 QString KateView::selectionText() const
 {
+  QMutexLocker l(m_doc->smartMutex());
+
   KTextEditor::Range range = *m_selection;
 
   if ( blockSelect )
@@ -1471,6 +1508,8 @@ QString KateView::selectionText() const
 
 bool KateView::removeSelectedText()
 {
+  QMutexLocker l(m_doc->smartMutex());
+
   if (!selection())
     return false;
 
@@ -1858,6 +1897,8 @@ bool KateView::blockSelectionMode () const
 
 bool KateView::setBlockSelectionMode (bool on)
 {
+  QMutexLocker l(m_doc->smartMutex());
+
   if (on != blockSelect)
   {
     blockSelect = on;
@@ -1870,12 +1911,9 @@ bool KateView::setBlockSelectionMode (bool on)
 
     m_toggleBlockSelection->setChecked( blockSelectionMode() );
 
-    if(!blockSelectionMode() && !m_viewInternal->getCursor().isValid())
-    {
-    	KTextEditor::Cursor cursorAtEndOfLine(cursorPosition());
-    	cursorAtEndOfLine.setColumn(m_doc->kateTextLine(cursorPosition().line())->length());
-    	setCursorPosition(cursorAtEndOfLine);
-    }
+    // when leaving block selection mode, if cursor is at an invalid position or past the end of the
+    // line, move the cursor to the last column of the current line unless cursor wrapping is off
+    ensureCursorColumnValid();
   }
 
   return true;
@@ -2064,17 +2102,17 @@ void KateView::unregisterCompletionModel(KTextEditor::CodeCompletionModel* model
 
 bool KateView::isAutomaticInvocationEnabled() const
 {
-  return completionWidget()->isAutomaticInvocationEnabled();
+  return m_config->automaticCompletionInvocation();
 }
 
 void KateView::setAutomaticInvocationEnabled(bool enabled)
 {
-  completionWidget()->setAutomaticInvocationEnabled(enabled);
+  config()->setAutomaticCompletionInvocation(enabled);
 }
 
-void KateView::sendCompletionExecuted(const KTextEditor::Cursor& position, KTextEditor::CodeCompletionModel* model, int row)
+void KateView::sendCompletionExecuted(const KTextEditor::Cursor& position, KTextEditor::CodeCompletionModel* model, const QModelIndex& index)
 {
-  emit completionExecuted(this, position, model, row);
+  emit completionExecuted(this, position, model, index);
 }
 
 void KateView::sendCompletionAborted()
@@ -2371,7 +2409,11 @@ void KateView::shiftToMatchingBracket( )
 
 const KTextEditor::Range & KateView::selectionRange( ) const
 {
-  return *m_selection;
+  QMutexLocker l(m_doc->smartMutex());
+
+  m_holdSelectionRangeForAPI = *m_selection;
+
+  return m_holdSelectionRangeForAPI;
 }
 
 KTextEditor::Document * KateView::document( ) const
@@ -2410,7 +2452,7 @@ QMenu * KateView::defaultContextMenu(QMenu* menu) const
     menu->addActions(popupwidget->actions());
 
   } else {
-    kDebug() << "no ktexteditor_popup container found; populating manually";
+    kDebug( 13030 ) << "no ktexteditor_popup container found; populating manually";
     menu->addAction(m_editUndo);
     menu->addAction(m_editRedo);
     menu->addSeparator();
@@ -2436,8 +2478,6 @@ void KateView::aboutToShowContextMenu( )
     emit contextMenuAboutToShow(this, menu);
 }
 
-//END Code completion new
-
 // BEGIN ConfigInterface stuff
 QStringList KateView::configKeys() const
 {
@@ -2487,6 +2527,11 @@ KateCmdLine *KateView::cmdLine ()
   return m_cmdLine = new KateCmdLine (this, m_viewBar);
 }
 
+KateViewBar *KateView::viewBar() const
+{
+  return m_viewBar;
+}
+
 KateSearchBar *KateView::searchBar (bool initHintAsPower)
 {
   if (m_searchBar)
@@ -2501,11 +2546,31 @@ KateGotoBar *KateView::gotoBar ()
     return m_gotoBar;
 
   m_gotoBar = new KateGotoBar (m_viewBar);
-#if QT_VERSION < KDE_MAKE_VERSION(4,4,0) // Remove this once Qt 4.4 becomes necessary
-  foreach (QWidget* widget, m_gotoBar->findChildren<QWidget*>())
-    actionCollection ()->addAssociatedWidget(widget);
-#endif
   return m_gotoBar;
 }
 
+void KateView::setAnnotationModel( KTextEditor::AnnotationModel* model )
+{
+  KTextEditor::AnnotationModel* oldmodel = m_annotationModel;
+  m_annotationModel = model;
+  m_viewInternal->m_leftBorder->annotationModelChanged(oldmodel, m_annotationModel);
+}
+
+KTextEditor::AnnotationModel* KateView::annotationModel() const
+{
+  return m_annotationModel;
+}
+
+void KateView::setAnnotationBorderVisible( bool visible )
+{
+  m_viewInternal->m_leftBorder->setAnnotationBorderOn( visible );
+}
+
+bool KateView::isAnnotationBorderVisible() const
+{
+  return m_viewInternal->m_leftBorder->annotationBorderOn();
+}
+
 // kate: space-indent on; indent-width 2; replace-tabs on;
+
+
