@@ -20,6 +20,8 @@
    Boston, MA 02110-1301, USA.
 */
 
+#include "kconfigbackend.h"
+
 #include <QtCore/QDateTime>
 #include <QtCore/QStringList>
 #include <QtCore/QDir>
@@ -33,7 +35,6 @@
 #include "kservicetypetrader.h"
 //#include "kapplication.h"
 #include "kconfig.h"
-#include "kconfigbackend.h"
 #include "kconfigini_p.h"
 #include "kconfigdata.h"
 #include "kdebug.h"
@@ -61,45 +62,28 @@ void KConfigBackend::registerMappings(const KEntryMap& /*entryMap*/)
 }
 
 BackendPtr KConfigBackend::create(const KComponentData& componentData, const QString& file,
-                                  const QString& system)
+                                  const QString& sys)
 {
-    const QString upperSystem = (system.isEmpty() ?
-            Private::whatSystem(file).toUpper() :
-            system.toUpper());
+    //qDebug() << "creating a backend for file" << file << "with system" << sys;
+    const QString system = (sys.isEmpty() ? Private::whatSystem(file) : sys);
     KConfigBackend* backend = 0;
 
-    if (upperSystem == "INI")
-        goto default_backend;
-    else {
-        KService::List offers = KServiceTypeTrader::self()->query("KConfigBackend");
+    if (system.compare("INI", Qt::CaseInsensitive) != 0) {
+        QString constraint = QString("'%1' ~~ Name").arg(system);
+        KService::List offers = KServiceTypeTrader::self()->query("KConfigBackend", constraint);
 
-        foreach (const KService::Ptr service, offers) {
-            if (service->name().toUpper() != upperSystem ||
-                service->library().isEmpty())
-                continue;
-
-            backend = KPluginLoader(*service, componentData).factory()->create<KConfigBackend>(QLatin1String("KConfigBackend"));
-//             const char* libraryName = service->library().toLocal8Bit().constData();
-//             int error = 0;
-
-/*            backend = KLibLoader::createInstance<KConfigBackend>(
-                                                   libraryName,
-                                                   0,
-                                                   QStringList(),
-                                                   &error);*/
+        //qDebug() << "found" << offers.count() << "offers for KConfigBackend plugins with name" << system;
+        foreach (const KService::Ptr& offer, offers) {
+            backend = offer->createInstance<KConfigBackend>(0);
             if (backend) {
+                //qDebug() << "successfully created a backend for" << system;
                 backend->setFilePath(file);
                 return BackendPtr(backend);
-            } else {
-                kDebug(181) << "Could not load config backend " << system
-                            /*<< ". Error code: " << error*/ << endl;
-                goto default_backend;
             }
         } // foreach offers
     }
 
-default_backend:
-
+    //qDebug() << "default creation of the Ini backend";
     backend = new KConfigIniBackend;
     backend->setFilePath(file);
     return BackendPtr(backend);
