@@ -336,6 +336,25 @@ void KUrlTest::testSimpleMethods() // to test parsing, mostly
 #endif
 }
 
+void KUrlTest::testHostName()
+{
+    KUrl u1("http://www.Abc.de/FR");
+    QCOMPARE(u1.host(), QString("www.abc.de")); // lowercase
+    QCOMPARE(u1.path(), QString("/FR")); // lowercase
+    QCOMPARE(u1.url(), QString("http://www.abc.de/FR")); // hostname is lowercase
+    QCOMPARE(u1.prettyUrl(), QString("http://www.abc.de/FR")); // hostname is lowercase
+
+    KUrl u2;
+    u2.setProtocol("http");
+    u2.setHost("www.Abc.de");
+    QCOMPARE(u2.host(), QString("www.abc.de")); // lowercase
+    QCOMPARE(u2.url(), QString("http://www.abc.de")); // lowercase
+
+    KUrl u3("donkey://Abc/DE");
+    QCOMPARE(u3.host(), QString("abc")); // lowercase
+    QCOMPARE(u3.url(), QString("donkey://abc/DE")); // lowercase
+}
+
 void KUrlTest::testEmptyQueryOrRef()
 {
   QUrl url = QUrl::fromEncoded( "http://www.kde.org" );
@@ -657,6 +676,11 @@ void KUrlTest::testIsRelative()
   // arguable, but necessary for KUrl( baseURL, "//www1.foo.bar" );
   QVERIFY( KUrl::isRelativeUrl("/path") );
   QVERIFY( KUrl::isRelativeUrl("something") );
+
+  KUrl something("something");
+  QCOMPARE(something.url(), QString("something"));
+  QCOMPARE(something.protocol(), QString());
+  QVERIFY(!something.isLocalFile());
 }
 
 void KUrlTest::testRelativePath()
@@ -1083,7 +1107,9 @@ void KUrlTest::testSetUser()
 
   KUrl emptyUserTest1( "http://www.foobar.com/");
   QVERIFY( emptyUserTest1.user().isEmpty() );
-  QVERIFY( !emptyUserTest1.user().isNull() ); // small change compared to KUrl-3.5
+#if QT_VERSION >= 0x040400
+  QVERIFY( emptyUserTest1.user().isNull() ); // Expected result. This was fixed in Qt-4.4
+#endif
   KUrl emptyUserTest2( "http://www.foobar.com/");
   emptyUserTest2.setUser( "" );
   //QVERIFY( emptyUserTest2.user().isNull() );
@@ -1231,11 +1257,29 @@ void KUrlTest::testBrokenStuff()
   QCOMPARE( broken.protocol(), QString("") ); // KDE3: was "file"
 #endif
 
+  {
+      QUrl url;
+      url.setEncodedUrl("LABEL=USB_STICK", QUrl::TolerantMode);
+      QVERIFY( url.isValid() );
+      QCOMPARE( url.path(), QString("LABEL=USB_STICK") );
+      QVERIFY( !url.isEmpty() );
+  }
+  {
+      QUrl url;
+      url.setEncodedUrl("LABEL=USB_STICK", QUrl::TolerantMode);
+      QVERIFY( url.isValid() );
+      QVERIFY( !url.isEmpty() ); // Qt-4.4-snapshot20080213 bug, reported to TT
+      QCOMPARE( url.path(), QString("LABEL=USB_STICK") );
+  }
+
   broken = "LABEL=USB_STICK"; // 71430, can we use KUrl for this?
   QVERIFY( broken.isValid() ); // KDE3 difference: QUrl likes this one too
   QVERIFY( !broken.isEmpty() );
   QCOMPARE( broken.path(), QString("LABEL=USB_STICK") ); // was "" in KDE3
+}
 
+void KUrlTest::testMoreBrokenStuff()
+{
 #if 0 // BROKEN?
   // UNC like names
   KUrl unc1("FILE://localhost/home/root");
@@ -1315,6 +1359,7 @@ void KUrlTest::testBrokenStuff()
   QCOMPARE( weird.path(), QString( ":keyword" ) );
   QCOMPARE( weird.url(), QString( ":keyword" ) ); // # BUG: the : is missing
 
+  KUrl broken;
   broken = "ptal://mlc:usb:PC_970";
   QVERIFY( !broken.isValid() );
   QEXPECT_FAIL( "", "QUrl doesn't provide the initial string if it's an invalid url...", Continue );
@@ -1352,10 +1397,24 @@ void KUrlTest::testBrokenStuff()
 
 void KUrlTest::testMailto()
 {
-  KUrl umail1 ( "mailto:faure@kde.org" );
+  const QString faure = "faure@kde.org";
+  const QString mailtoFaure = "mailto:" + faure;
+  KUrl umail1 ( mailtoFaure );
   QCOMPARE( umail1.protocol(), QString("mailto") );
-  QCOMPARE( umail1.path(), QString("faure@kde.org") );
-  QVERIFY( !KUrl::isRelativeUrl("mailto:faure@kde.org") );
+  QCOMPARE( umail1.path(), QString(faure) );
+  QVERIFY( !KUrl::isRelativeUrl(mailtoFaure) );
+
+  // Make sure populateMimeData() works correct:
+  // 1. the text/plain part of the mimedata should not contain the mailto: part
+  // 2. the uri-list part of the mimedata should contain the mailto: part
+  QMimeData md;
+  umail1.populateMimeData(&md);
+  QCOMPARE(md.text(), faure);
+  KUrl::List uriList = KUrl::List::fromMimeData(&md);
+  QCOMPARE(uriList.size(), 1);
+  KUrl first = uriList.first();
+  QCOMPARE(first.protocol(), QString("mailto"));
+  QCOMPARE(first.path(), faure);
 
   KUrl mailtoOnly( "mailto:" );
   QVERIFY( mailtoOnly.isValid() ); // KDE3 said invalid, QUrl is more tolerant
@@ -1393,8 +1452,12 @@ void KUrlTest::testSmb()
 
 void KUrlTest::testOtherProtocols()
 {
-  KUrl about("about:konqueror");
-  QCOMPARE(about.path(),QString("konqueror") );
+  KUrl about("about:");
+  QCOMPARE(about.path(), QString());
+  QCOMPARE(about.protocol(), QString("about"));
+
+  KUrl aboutKonqueror("about:konqueror");
+  QCOMPARE(aboutKonqueror.path(), QString("konqueror"));
 
 
   KUrl leo( "data:text/html,http://www.invalid/" );
@@ -1444,6 +1507,10 @@ void KUrlTest::testUtf8()
 
   KUrl umlaut2("http://www.clever-tanken.de/liste.asp?ort=N%FCrnberg&typ=Diesel"); // was ,106
   QCOMPARE(umlaut2.url(), QString("http://www.clever-tanken.de/liste.asp?ort=N%FCrnberg&typ=Diesel"));
+
+  KUrl wkai(QString::fromUtf8("/tmp/魔"));
+  QCOMPARE(wkai.url(), QString("file:///tmp/%E9%AD%94"));
+  QCOMPARE(wkai.prettyUrl(), QString::fromUtf8("file:///tmp/魔"));
 }
 
 void KUrlTest::testOtherEncodings()
@@ -1579,12 +1646,20 @@ void KUrlTest::testQueryItem()
 void KUrlTest::testEncodeString()
 {
   // Needed for #49616
-  QCOMPARE( KUrl::toPercentEncoding( "C++" ), QByteArray("C%2B%2B") );
-  QCOMPARE( KUrl::fromPercentEncoding( "C%2B%2B" ), QString("C++") );
-  QCOMPARE( KUrl::fromPercentEncoding( "C%00%A" ), QString("C") ); // we stop at %00
+  QCOMPARE( QUrl::toPercentEncoding( "C++" ), QByteArray("C%2B%2B") );
+  QCOMPARE( QUrl::fromPercentEncoding( "C%2B%2B" ), QString("C++") );
+#if QT_VERSION < 0x040400
+  QCOMPARE( QUrl::fromPercentEncoding( "C%00%A" ), QString("C") ); // stop at %00, like in kde3
+#else
+  QString output = QUrl::fromPercentEncoding( "C%00%0A" );
+  QString expected = QString::fromLatin1("C\0\n", 3); // no reason to stop at %00, in fact
+  QCOMPARE( output.size(), expected.size() );
+  QCOMPARE( output, expected );
+#endif
+  QCOMPARE( QUrl::fromPercentEncoding( "C%A" ), QString("C%A") ); // % A is not percent-encoding  (pct-encoded = "%" HEXDIG HEXDIG)
 
-  QCOMPARE( KUrl::toPercentEncoding( "%" ), QByteArray("%25") );
-  QCOMPARE( KUrl::toPercentEncoding( ":" ), QByteArray("%3A") );
+  QCOMPARE( QUrl::toPercentEncoding( "%" ), QByteArray("%25") );
+  QCOMPARE( QUrl::toPercentEncoding( ":" ), QByteArray("%3A") );
 }
 
 void KUrlTest::testIdn()
