@@ -19,6 +19,7 @@
 */
 #include <kedittoolbar.h>
 #include <kedittoolbar_p.h>
+#include <QShowEvent>
 
 
 #include <QtXml/QDomDocument>
@@ -26,7 +27,6 @@
 #include <QtCore/QDir>
 #include <QtCore/QFile>
 #include <QHeaderView>
-#include <QtGui/QComboBox>
 #include <QtGui/QToolButton>
 #include <QtGui/QLabel>
 #include <QtGui/QApplication>
@@ -46,6 +46,7 @@
 #include <kprocess.h>
 #include <ktoolbar.h>
 #include <kdeversion.h>
+#include <kcombobox.h>
 
 #include "kaction.h"
 #include "kactioncollection.h"
@@ -378,7 +379,7 @@ public:
     }
 #endif
 
-    QComboBox *m_toolbarCombo;
+    KComboBox *m_toolbarCombo;
 
     QToolButton *m_upAction;
     QToolButton *m_removeAction;
@@ -523,7 +524,7 @@ void KEditToolBarPrivate::_k_acceptOK(bool b)
 
 void KEditToolBarPrivate::_k_slotDefault()
 {
-    if ( KMessageBox::warningContinueCancel(q, i18n("Do you really want to reset all toolbars of this application to their default? The changes will be applied immediately."), i18n("Reset ToolBars"),KGuiItem(i18n("Reset")))!=KMessageBox::Continue )
+    if ( KMessageBox::warningContinueCancel(q, i18n("Do you really want to reset all toolbars of this application to their default? The changes will be applied immediately."), i18n("Reset Toolbars"),KGuiItem(i18n("Reset")))!=KMessageBox::Continue )
         return;
 
     delete m_widget;
@@ -736,11 +737,7 @@ void KEditToolBarWidgetPrivate::initKPart( KXMLGUIFactory* factory,
 
   m_widget->actionCollection()->addAssociatedWidget( m_widget );
   foreach (QAction* action, m_widget->actionCollection()->actions())
-#if QT_VERSION < KDE_MAKE_VERSION(4,4,0)
-    action->setShortcutContext(Qt::WidgetShortcut); // remove after Qt4.4 becomes mandatory
-#else
     action->setShortcutContext(Qt::WidgetWithChildrenShortcut);
-#endif
 }
 
 bool KEditToolBarWidget::save()
@@ -825,7 +822,7 @@ void KEditToolBarWidgetPrivate::setupLayout()
 {
   // the toolbar name combo
   m_comboLabel = new QLabel(i18n("&Toolbar:"), m_widget);
-  m_toolbarCombo = new QComboBox(m_widget);
+  m_toolbarCombo = new KComboBox(m_widget);
   m_comboLabel->setBuddy(m_toolbarCombo);
   m_comboSeparator = new KSeparator(m_widget);
   QObject::connect(m_toolbarCombo, SIGNAL(activated(int)),
@@ -844,6 +841,7 @@ void KEditToolBarWidgetPrivate::setupLayout()
   m_inactiveList->setDragEnabled(true);
   m_inactiveList->setActiveList(false);
   m_inactiveList->setMinimumSize(180, 250);
+  m_inactiveList->setDropIndicatorShown(false); // #165663
   inactive_label->setBuddy(m_inactiveList);
   QObject::connect(m_inactiveList, SIGNAL(itemSelectionChanged()),
                    m_widget,       SLOT(slotInactiveSelectionChanged()));
@@ -1022,7 +1020,7 @@ void KEditToolBarWidgetPrivate::loadActions(const QDomElement& elem)
   KActionCollection* actionCollection = m_currentXmlData->actionCollection();
 
   // store the names of our active actions
-  QMap<QString, bool> active_list;
+  QSet<QString> active_list;
 
   // i18n filtering message for action names
   KLocalizedString nameFilter = ki18nc("@item:intable Action name in toolbar editor", "%1");
@@ -1074,7 +1072,7 @@ void KEditToolBarWidgetPrivate::loadActions(const QDomElement& elem)
         act->setText(nameFilter.subs(action->text().remove(QChar('&'))).toString());
         act->setIcon(!action->icon().isNull() ? action->icon() : m_emptyIcon);
 
-        active_list.insert(action->objectName(), true);
+        active_list.insert(action->objectName());
         break;
       }
     }
@@ -1521,16 +1519,18 @@ void KEditToolBarWidgetPrivate::slotDropped(ToolBarListWidget* list, int index,
 
 void KEditToolBar::showEvent( QShowEvent * event )
 {
-  // The dialog has been shown, enable toolbar editing
-    if ( d->m_factory ) {
-        // call the kpart version
-        d->m_widget->load( d->m_factory, d->m_defaultToolBar );
-    } else {
-        // call the action collection version
-        d->m_widget->load( d->m_file, d->m_global, d->m_defaultToolBar );
-    }
+    if (!event->spontaneous()) {
+        // The dialog has been shown, enable toolbar editing
+        if ( d->m_factory ) {
+            // call the kpart version
+            d->m_widget->load( d->m_factory, d->m_defaultToolBar );
+        } else {
+            // call the action collection version
+            d->m_widget->load( d->m_file, d->m_global, d->m_defaultToolBar );
+        }
 
-    KToolBar::setToolBarsEditable(true);
+        KToolBar::setToolBarsEditable(true);
+    }
     KDialog::showEvent(event);
 }
 
