@@ -256,6 +256,7 @@ bool KWindowSystemPrivate::mapViewport()
 // compiz claims support even though it doesn't use virtual desktops :(
 //    if( isSupported( NET::DesktopViewport ) && !isSupported( NET::NumberOfDesktops ))
 
+// this test is duplicated in KWindowSystem::mapViewport()
     if( isSupported( NET::DesktopViewport ) && numberOfDesktops( true ) <= 1
         && ( desktopGeometry( currentDesktop( true )).width > QApplication::desktop()->width()
             || desktopGeometry( currentDesktop( true )).height > QApplication::desktop()->height()))
@@ -337,11 +338,11 @@ void KWindowSystem::connectNotify( const char* signal )
         what = INFO_WINDOWS;
     else if( QLatin1String( signal ) == SIGNAL(strutChanged()))
         what = INFO_WINDOWS;
-    else if( QLatin1String( signal ) == QMetaObject::normalizedSignature(SIGNAL(windowChanged(WId,const unsigned long*))).data())
+    else if( QLatin1String( signal ) == QMetaObject::normalizedSignature(SIGNAL(windowChanged(WId,const unsigned long*))).constData())
         what = INFO_WINDOWS;
-    else if( QLatin1String( signal ) ==  QMetaObject::normalizedSignature(SIGNAL(windowChanged(WId,unsigned int))).data())
+    else if( QLatin1String( signal ) ==  QMetaObject::normalizedSignature(SIGNAL(windowChanged(WId,unsigned int))).constData())
         what = INFO_WINDOWS;
-    else if( QLatin1String( signal ) ==  QMetaObject::normalizedSignature(SIGNAL(windowChanged(WId))).data())
+    else if( QLatin1String( signal ) ==  QMetaObject::normalizedSignature(SIGNAL(windowChanged(WId))).constData())
         what = INFO_WINDOWS;
 
     init( what );
@@ -413,9 +414,9 @@ int KWindowSystem::currentDesktop()
 
     KWindowSystemPrivate* const s_d = s_d_func();
     if( s_d )
-        return s_d->currentDesktop();
+        return s_d->currentDesktop( true );
     NETRootInfo info( QX11Info::display(), NET::CurrentDesktop );
-    return info.currentDesktop();
+    return info.currentDesktop( true );
 }
 
 int KWindowSystem::numberOfDesktops()
@@ -432,9 +433,9 @@ int KWindowSystem::numberOfDesktops()
 
     KWindowSystemPrivate* const s_d = s_d_func();
     if( s_d )
-        return s_d->numberOfDesktops();
+        return s_d->numberOfDesktops( true );
     NETRootInfo info( QX11Info::display(), NET::NumberOfDesktops );
-    return info.numberOfDesktops();
+    return info.numberOfDesktops( true );
 }
 
 void KWindowSystem::setCurrentDesktop( int desktop )
@@ -451,7 +452,7 @@ void KWindowSystem::setCurrentDesktop( int desktop )
         return;
     }
     NETRootInfo info( QX11Info::display(), 0 );
-    info.setCurrentDesktop( desktop );
+    info.setCurrentDesktop( desktop, true );
 }
 
 void KWindowSystem::setOnAllDesktops( WId win, bool b )
@@ -465,10 +466,10 @@ void KWindowSystem::setOnAllDesktops( WId win, bool b )
     }
     NETWinInfo info( QX11Info::display(), win, QX11Info::appRootWindow(), NET::WMDesktop );
     if ( b )
-	info.setDesktop( NETWinInfo::OnAllDesktops );
-    else if ( info.desktop()  == NETWinInfo::OnAllDesktops ) {
+	info.setDesktop( NETWinInfo::OnAllDesktops, true );
+    else if ( info.desktop( true )  == NETWinInfo::OnAllDesktops ) {
 	NETRootInfo rinfo( QX11Info::display(), NET::CurrentDesktop );
-	info.setDesktop( rinfo.currentDesktop() );
+	info.setDesktop( rinfo.currentDesktop( true ), true );
     }
 }
 
@@ -507,7 +508,7 @@ void KWindowSystem::setOnDesktop( WId win, int desktop )
         return;
     }
     NETWinInfo info( QX11Info::display(), win, QX11Info::appRootWindow(), NET::WMDesktop );
-    info.setDesktop( desktop );
+    info.setDesktop( desktop, true );
 }
 
 WId KWindowSystem::activeWindow()
@@ -860,12 +861,12 @@ void KWindowSystem::setDesktopName( int desktop, const QString& name )
         desktop = currentDesktop();
 
     if( s_d ) {
-        s_d->setDesktopName( desktop, name.toUtf8().data() );
+        s_d->setDesktopName( desktop, name.toUtf8().constData() );
         return;
     }
 
     NETRootInfo info( QX11Info::display(), 0 );
-    info.setDesktopName( desktop, name.toUtf8().data() );
+    info.setDesktopName( desktop, name.toUtf8().constData() );
 }
 
 bool KWindowSystem::showingDesktop()
@@ -966,8 +967,19 @@ void KWindowSystem::doNotManage( const QString& title )
 
 bool KWindowSystem::mapViewport()
 {
-    init( INFO_BASIC );
-    return s_d_func()->mapViewport();
+    KWindowSystemPrivate* const s_d = s_d_func();
+    if( s_d )
+        return s_d->mapViewport();
+    // avoid creating KWindowSystemPrivate
+    NETRootInfo infos( QX11Info::display(), NET::Supported );
+    if( !infos.isSupported( NET::DesktopViewport ))
+        return false;
+    NETRootInfo info( QX11Info::display(), NET::NumberOfDesktops | NET::CurrentDesktop | NET::DesktopGeometry );
+    if( info.numberOfDesktops( true ) <= 1
+        && ( info.desktopGeometry( info.currentDesktop( true )).width > QApplication::desktop()->width()
+            || info.desktopGeometry( info.currentDesktop( true )).height > QApplication::desktop()->height()))
+        return true;
+    return false;
 }
 
 int KWindowSystem::viewportToDesktop( const QPoint& p )
