@@ -92,21 +92,21 @@ namespace KJS {
            FormAction, FormEncType, FormElements, FormLength, FormAcceptCharset,
            FormReset, FormTarget, FormName, FormMethod, FormSubmit, SelectAdd,
            SelectTabIndex, SelectValue, SelectSelectedIndex, SelectLength,
-           SelectRemove, SelectForm, SelectBlur, SelectType, SelectOptions,
-           SelectDisabled, SelectMultiple, SelectName, SelectSize, SelectFocus,
+           SelectRemove, SelectForm, SelectType, SelectOptions,
+           SelectDisabled, SelectMultiple, SelectName, SelectSize, 
            OptGroupDisabled, OptGroupLabel, OptionIndex, OptionSelected,
            OptionForm, OptionText, OptionDefaultSelected, OptionDisabled,
-           OptionLabel, OptionValue, InputBlur, InputReadOnly, InputAccept,
+           OptionLabel, OptionValue, InputReadOnly, InputAccept,
            InputSize, InputDefaultValue, InputTabIndex, InputValue, InputType,
-           InputFocus, InputMaxLength, InputDefaultChecked, InputDisabled,
+           InputMaxLength, InputDefaultChecked, InputDisabled,
            InputChecked, InputIndeterminate, InputForm, InputAccessKey, InputAlign, InputAlt,
            InputName, InputSrc, InputUseMap, InputSelect, InputClick,
            InputSelectionStart, InputSelectionEnd, InputSetSelectionRange,
            TextAreaAccessKey, TextAreaName, TextAreaDefaultValue, TextAreaSelect,
            TextAreaCols, TextAreaDisabled, TextAreaForm, TextAreaType,
            TextAreaTabIndex, TextAreaReadOnly, TextAreaRows, TextAreaValue,
-           TextAreaBlur, TextAreaFocus, TextAreaSelectionStart, TextAreaSelectionEnd, TextAreaSetSelectionRange,
-           TextAreaTextLength, ButtonBlur, ButtonFocus, ButtonClick, ButtonForm, ButtonTabIndex, ButtonName,
+           TextAreaSelectionStart, TextAreaSelectionEnd, TextAreaSetSelectionRange,
+           TextAreaTextLength, ButtonClick, ButtonForm, ButtonTabIndex, ButtonName,
            ButtonDisabled, ButtonAccessKey, ButtonType, ButtonValue, LabelHtmlFor,
            LabelForm, LabelAccessKey, FieldSetForm, LegendForm, LegendAccessKey,
            LegendAlign, UListType, UListCompact, OListStart, OListCompact,
@@ -116,9 +116,9 @@ namespace KJS {
            BaseFontFace, FontColor, FontSize, FontFace, HRWidth, HRNoShade,
            HRAlign, HRSize, ModCite, ModDateTime, AnchorShape, AnchorRel,
            AnchorAccessKey, AnchorCoords, AnchorHref, AnchorProtocol, AnchorHost,
-           AnchorCharset, AnchorHrefLang, AnchorHostname, AnchorType, AnchorFocus,
+           AnchorCharset, AnchorHrefLang, AnchorHostname, AnchorType,
            AnchorPort, AnchorPathName, AnchorHash, AnchorSearch, AnchorName,
-           AnchorRev, AnchorTabIndex, AnchorTarget, AnchorText, AnchorBlur, AnchorClick,
+           AnchorRev, AnchorTabIndex, AnchorTarget, AnchorText, AnchorClick,
            ImageName, ImageAlign, ImageHspace, ImageVspace, ImageUseMap, ImageAlt,
            ImageLowSrc, ImageWidth, ImageIsMap, ImageBorder, ImageHeight,
            ImageLongDesc, ImageSrc, ImageX, ImageY, ImageComplete, ObjectHspace, ObjectHeight, ObjectAlign,
