@@ -175,6 +175,8 @@ public:
     virtual void defaultEventHandler(EventImpl *evt);
     virtual bool isEditable();
 
+    virtual bool unsubmittedFormChanges() const { return false; }
+
 protected:
     HTMLFormElementImpl *getForm() const;
 
@@ -209,8 +211,6 @@ public:
     void activate();
     virtual void attach();
     void click();
-    void blur();
-    void focus();
 
 protected:
     DOMString             m_value;
@@ -283,9 +283,6 @@ public:
 
     DOMString valueWithDefault() const;
 
-    void blur();
-    void focus();
-
     virtual bool maintainsState() { return true; }
     virtual QString state();
     virtual void restoreState(const QString &state);
@@ -314,6 +311,7 @@ public:
     void activate();
 
     void setUnsubmittedFormChange(bool unsubmitted) { m_unsubmittedFormChange = unsubmitted; }
+    virtual bool unsubmittedFormChanges() const { return m_unsubmittedFormChange; }
 
     //Mozilla extensions.
     long selectionStart();
@@ -407,8 +405,6 @@ public:
 
     void add ( HTMLElementImpl* element, HTMLElementImpl* before, int& exceptioncode );
     void remove ( long index );
-    void blur();
-    void focus();
 
     DOMString value() const;
     void setValue(DOMStringImpl* value);
@@ -566,11 +562,10 @@ public:
     void setValue(DOMString _value);
     DOMString defaultValue();
     void setDefaultValue(DOMString _defaultValue);
-    void blur();
-    void focus();
 
     virtual bool isEditable();
     void setUnsubmittedFormChange(bool unsubmitted) { m_unsubmittedFormChange = unsubmitted; }
+    virtual bool unsubmittedFormChanges() const { return m_unsubmittedFormChange; }
 
     //Mozilla extensions.
     long selectionStart();
