@@ -125,6 +125,10 @@ bool DOMImplementationImpl::hasFeature ( const DOMString &feature, const DOMStri
         (version.isEmpty() || version == "2.0"))
         return true;
 
+    if ((lower == "css") &&
+        (version.isEmpty() || version == "2.0"))
+        return true;
+
     if ((lower == "events" || lower == "uievents" ||
          lower == "mouseevents" || lower == "mutationevents" ||
          lower == "htmlevents" || lower == "textevents" ) &&
@@ -1162,9 +1166,11 @@ QStringList DocumentImpl::docState()
 
 bool DocumentImpl::unsubmittedFormChanges()
 {
-    for (QListIterator<NodeImpl*> it(m_maintainsState); it.hasNext();)
-        if (it.next()->state().endsWith('M'))
+    for (QListIterator<NodeImpl*> it(m_maintainsState); it.hasNext();) {
+        NodeImpl* node = it.next();
+        if (node->isGenericFormElement() && static_cast<HTMLGenericFormElementImpl*>(node)->unsubmittedFormChanges())
             return true;
+    }
 
     return false;
 }
@@ -1436,6 +1442,7 @@ void DocumentImpl::open( bool clearEventListeners )
         detach();
 
     removeChildren();
+    childrenChanged(); // Reset m_documentElement
     delete m_styleSelector;
     m_styleSelector = 0;
     m_view = view;
@@ -1829,7 +1836,7 @@ void DocumentImpl::processHttpEquiv(const DOMString &equiv, const DOMString &con
     {
         QString str = content.string().toLower().trimmed();
         KUrl url = v->part()->url();
-        if ((str == "no-cache") && url.protocol().startsWith("http"))
+        if ((str == "no-cache") && url.protocol().startsWith(QLatin1String("http")))
         {
            KIO::http_update_cache(url, true, 0);
         }
@@ -2400,6 +2407,11 @@ void DocumentImpl::setFocusNode(NodeImpl *newFocusNode)
     // don't process focus changes while detaching
     if( !m_render ) return;
 
+    // See if the new node is really focusable. It might not be
+    // if focus() was called explicitly.
+    if (newFocusNode && !newFocusNode->isFocusable())
+        return;
+
     // Make sure newFocusNode is actually in this document
     if (newFocusNode && (newFocusNode->document() != this))
         return;
@@ -2414,6 +2426,12 @@ void DocumentImpl::setFocusNode(NodeImpl *newFocusNode)
                 oldFocusNode->setActive(false);
 
             oldFocusNode->setFocus(false);
+            if (oldFocusNode->renderer() && oldFocusNode->renderer()->isWidget()) {
+                // Editable widgets may need to dispatch CHANGE_EVENT
+                RenderWidget* rw = static_cast<RenderWidget*>(oldFocusNode->renderer());
+                if (rw->isRedirectedWidget())
+                    rw->handleFocusOut();
+            }
 
             oldFocusNode->dispatchHTMLEvent(EventImpl::BLUR_EVENT,false,false);
             oldFocusNode->dispatchUIEvent(EventImpl::DOMFOCUSOUT_EVENT);
