@@ -64,6 +64,10 @@ public:
                      bool delayedMimeTypes)
         : m_entry( entry ),
           m_url( url ),
+          m_strName(),
+          m_strText(),
+          m_iconName(),
+          m_strLowerCaseName(),
           m_pMimeType( 0 ),
           m_fileMode( mode ),
           m_permissions( permissions ),
@@ -72,6 +76,7 @@ public:
           m_bIsLocalUrl( url.isLocalFile() ),
           m_bMimeTypeKnown( false ),
           m_delayedMimeTypes( delayedMimeTypes ),
+          m_useIconNameCache(false),
           m_hidden( Auto )
     {
         if (entry.count() != 0) {
@@ -99,6 +104,7 @@ public:
 
     KIO::filesize_t size() const;
     KDateTime time( KFileItem::FileTimes which ) const;
+    void setTime(KFileItem::FileTimes which, long long time_t_val) const;
     bool cmp( const KFileItemPrivate & item ) const;
 
     /**
@@ -132,6 +138,11 @@ public:
      */
     QString m_strText;
 
+    /**
+     * The icon name for this item.
+     */
+    mutable QString m_iconName;
+
     /**
      * The filename in lower case (to speed up sorting)
      */
@@ -167,6 +178,9 @@ public:
     mutable bool m_bMimeTypeKnown:1;
     bool m_delayedMimeTypes:1;
 
+    /** True if m_iconName should be used as cache. */
+    mutable bool m_useIconNameCache:1;
+
     // Auto: check leading dot.
     enum { Auto, Hidden, Shown } m_hidden:3;
 
@@ -200,7 +214,7 @@ void KFileItemPrivate::init()
              * This is the reason for the -1
              */
             KDE_struct_stat buf;
-            QByteArray path = QFile::encodeName(m_url.path( KUrl::RemoveTrailingSlash ));
+            const QByteArray path = QFile::encodeName(m_url.path( KUrl::RemoveTrailingSlash ));
             if ( KDE_lstat( path.data(), &buf ) == 0 )
             {
                 mode = buf.st_mode;
@@ -213,8 +227,8 @@ void KFileItemPrivate::init()
                         mode = (S_IFMT-1) | S_IRWXU | S_IRWXG | S_IRWXO;
                 }
                 // While we're at it, store the times
-                m_time[ KFileItem::ModificationTime ].setTime_t(buf.st_mtime);
-                m_time[ KFileItem::AccessTime ].setTime_t(buf.st_atime);
+                setTime(KFileItem::ModificationTime, buf.st_mtime);
+                setTime(KFileItem::AccessTime, buf.st_atime);
                 if ( m_fileMode == KFileItem::Unknown )
                     m_fileMode = mode & S_IFMT; // extract file type
                 if ( m_permissions == KFileItem::Unknown )
@@ -231,7 +245,13 @@ void KFileItemPrivate::readUDSEntry( bool _urlIsDirectory )
     m_fileMode = m_entry.numberValue( KIO::UDSEntry::UDS_FILE_TYPE );
     m_permissions = m_entry.numberValue( KIO::UDSEntry::UDS_ACCESS );
     m_strName = m_entry.stringValue( KIO::UDSEntry::UDS_NAME );
-    m_strText = KIO::decodeFileName( m_strName );
+
+    const QString displayName = m_entry.stringValue( KIO::UDSEntry::UDS_DISPLAY_NAME );
+    if (!displayName.isEmpty())
+      m_strText = displayName;
+    else
+      m_strText = KIO::decodeFileName( m_strName );
+
     const QString urlStr = m_entry.stringValue( KIO::UDSEntry::UDS_URL );
     const bool UDS_URL_seen = !urlStr.isEmpty();
     if ( UDS_URL_seen ) {
@@ -254,6 +274,8 @@ void KFileItemPrivate::readUDSEntry( bool _urlIsDirectory )
     static const QString& dot = KGlobal::staticQString(".");
     if ( _urlIsDirectory && !UDS_URL_seen && !m_strName.isEmpty() && m_strName != dot )
         m_url.addPath( m_strName );
+
+    m_iconName.clear();
 }
 
 inline //because it is used only in one place
@@ -274,6 +296,12 @@ KIO::filesize_t KFileItemPrivate::size() const
     return 0;
 }
 
+void KFileItemPrivate::setTime(KFileItem::FileTimes mappedWhich, long long time_t_val) const
+{
+    m_time[mappedWhich].setTime_t(time_t_val);
+    m_time[mappedWhich] = m_time[mappedWhich].toLocalZone(); // #160979
+}
+
 KDateTime KFileItemPrivate::time( KFileItem::FileTimes mappedWhich ) const
 {
     if ( !m_time[mappedWhich].isNull() )
@@ -293,7 +321,7 @@ KDateTime KFileItemPrivate::time( KFileItem::FileTimes mappedWhich ) const
         break;
     }
     if ( fieldVal != -1 ) {
-        m_time[mappedWhich].setTime_t( fieldVal );
+        setTime(mappedWhich, fieldVal);
         return m_time[mappedWhich];
     }
 
@@ -303,8 +331,8 @@ KDateTime KFileItemPrivate::time( KFileItem::FileTimes mappedWhich ) const
         KDE_struct_stat buf;
         if ( KDE_stat( QFile::encodeName(m_url.path(KUrl::RemoveTrailingSlash)), &buf ) == 0 )
         {
-            m_time[KFileItem::ModificationTime].setTime_t(buf.st_mtime);
-            m_time[KFileItem::AccessTime].setTime_t(buf.st_atime);
+            setTime(KFileItem::ModificationTime, buf.st_mtime);
+            setTime(KFileItem::AccessTime, buf.st_atime);
             m_time[KFileItem::CreationTime] = KDateTime();
             return m_time[mappedWhich];
         }
@@ -439,9 +467,9 @@ void KFileItem::refresh()
 {
     d->m_fileMode = KFileItem::Unknown;
     d->m_permissions = KFileItem::Unknown;
-    d->m_pMimeType = 0;
     d->m_metaInfo = KFileMetaInfo();
     d->m_hidden = KFileItemPrivate::Auto;
+    refreshMimeType();
 
     // Basically, we can't trust any information we got while listing.
     // Everything could have changed...
@@ -455,6 +483,7 @@ void KFileItem::refreshMimeType()
 {
     d->m_pMimeType = 0;
     d->m_bMimeTypeKnown = false;
+    d->m_iconName.clear();
 }
 
 void KFileItem::setUrl( const KUrl &url )
@@ -693,34 +722,44 @@ static QString iconFromDesktopFile(const QString& path)
 
 QString KFileItem::iconName() const
 {
-    const QString iconName = d->m_entry.stringValue( KIO::UDSEntry::UDS_ICON_NAME );
-    if (!iconName.isEmpty())
-        return iconName;
+    if (d->m_useIconNameCache && !d->m_iconName.isEmpty()) {
+        return d->m_iconName;
+    }
+
+    d->m_iconName = d->m_entry.stringValue( KIO::UDSEntry::UDS_ICON_NAME );
+    if (!d->m_iconName.isEmpty()) {
+        d->m_useIconNameCache = d->m_bMimeTypeKnown;
+        return d->m_iconName;
+    }
 
     bool isLocalUrl;
     KUrl url = mostLocalUrl(isLocalUrl);
 
     KMimeType::Ptr mime = mimeTypePtr();
     if (isLocalUrl && mime->is("application/x-desktop")) {
-        QString icon = iconFromDesktopFile(url.path());
-        if (!icon.isEmpty())
-            return icon;
+        d->m_iconName = iconFromDesktopFile(url.path());
+        if (!d->m_iconName.isEmpty()) {
+            d->m_useIconNameCache = d->m_bMimeTypeKnown;
+            return d->m_iconName;
+        }
     }
 
     //kDebug() << "finding icon for " << url.url() << " : " << mimeTypePtr()->name();
-    return mime->iconName(url);
+    d->m_iconName = mime->iconName(url);
+    d->m_useIconNameCache = d->m_bMimeTypeKnown;
+    return d->m_iconName;
 }
 
 QStringList KFileItem::overlays() const
 {
     QStringList names;
     if ( d->m_bLink ) {
-        names.append("link");
+        names.append("emblem-symbolic-link");
     }
 
     if ( !S_ISDIR( d->m_fileMode ) // Locked dirs have a special icon, use the overlay for files only
          && !isReadable()) {
-        names.append("lock");
+        names.append("object-locked");
     }
 
     if ( isHidden() ) {
@@ -733,14 +772,15 @@ QStringList KFileItem::overlays() const
         if (KSambaShare::instance()->isDirectoryShared( d->m_url.path() ) ||
             KNFSShare::instance()->isDirectoryShared( d->m_url.path() ))
         {
-            //kDebug()<<"KFileShare::isDirectoryShared : "<<d->m_url.path();
-            names.append("share");
+            //kDebug() << d->m_url.path();
+            names.append("network-workgroup");
         }
     }
 #endif  // Q_OS_WIN
 
-    if ( d->m_pMimeType && d->m_pMimeType->is("application/x-gzip") && d->m_url.fileName().endsWith( QLatin1String( ".gz" ) ) ) {
-        names.append("zio");
+    if ( d->m_pMimeType && d->m_url.fileName().endsWith( QLatin1String( ".gz" ) ) &&
+         d->m_pMimeType->is("application/x-gzip") ) {
+        names.append("application-zip");
     }
 
     return names;
@@ -783,7 +823,7 @@ QPixmap KFileItem::pixmap( int _size, int _state ) const
     {
         KUrl sf;
         sf.setPath( d->m_url.path().left( d->m_url.path().length() - 3 ) );
-        //kDebug() << "KFileItem::pixmap subFileName=" << subFileName;
+        //kDebug() << "subFileName=" << subFileName;
         mime = KMimeType::findByUrl( sf, 0, d->m_bIsLocalUrl );
     }
 
@@ -863,7 +903,7 @@ bool KFileItem::isDir() const
 {
     if ( d->m_fileMode == KFileItem::Unknown )
     {
-        kDebug() << " KFileItem::isDir can't say -> false ";
+        kDebug(7101) << "can't say -> false";
         return false; // can't say for sure, so no
     }
     return (S_ISDIR(d->m_fileMode));
@@ -916,6 +956,10 @@ QString KFileItem::getStatusBarInfo() const
         else
             text += i18n("(%1, Link to %2)", comment, linkDest());
     }
+    else if ( targetUrl() != url() )
+    {
+        text += i18n ( " (Points to %1)", targetUrl().pathOrUrl());
+    }
     else if ( S_ISREG( d->m_fileMode ) )
     {
         text += QString(" (%1, %2)").arg( comment, KIO::convertSize( size() ) );
@@ -968,10 +1012,10 @@ QString KFileItem::getToolTipText(int maxcount) const
     if (info.isValid())
     {
         tip += "<tr><td colspan=2><center><s>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</s></center></td></tr>";
-        QStringList keys = info.preferredKeys();
+        const QStringList keys = info.preferredKeys();
 
         // now the rest
-        QStringList::Iterator it = keys.begin();
+        QStringList::ConstIterator it = keys.begin();
         for (int count = 0; count<maxcount && it!=keys.end() ; ++it)
         {
             KFileMetaInfoItem item = info.item( *it );
@@ -1032,6 +1076,7 @@ void KFileItem::setUDSEntry( const KIO::UDSEntry& _entry, const KUrl& _url,
     d->m_url = _url;
     d->m_strName.clear();
     d->m_strText.clear();
+    d->m_iconName.clear();
     d->m_strLowerCaseName.clear();
     d->m_pMimeType = 0;
     d->m_fileMode = KFileItem::Unknown;
@@ -1044,6 +1089,7 @@ void KFileItem::setUDSEntry( const KIO::UDSEntry& _entry, const KUrl& _url,
     d->m_guessedMimeType.clear();
     d->m_metaInfo = KFileMetaInfo();
     d->m_delayedMimeTypes = _delayedMimeTypes;
+    d->m_useIconNameCache = false;
 
     d->readUDSEntry( _urlIsDirectory );
     d->init();
@@ -1200,6 +1246,15 @@ QString KFileItem::name( bool lowerCase ) const
     return d->m_strLowerCaseName;
 }
 
+KUrl KFileItem::targetUrl() const
+{
+    const QString targetUrlStr = d->m_entry.stringValue( KIO::UDSEntry::UDS_TARGET_URL );
+    if (!targetUrlStr.isEmpty())
+      return KUrl(targetUrlStr);
+    else
+      return url();
+}
+
 /*
  * Mimetype handling.
  *
@@ -1305,3 +1360,23 @@ KUrl::List KFileItemList::urlList() const {
     return lst;
 }
 
+bool KFileItem::isDesktopFile() const
+{
+    // only local files
+    bool isLocal;
+    const KUrl url = mostLocalUrl(isLocal);
+    if (!isLocal)
+        return false;
+
+    // only regular files
+    if (!S_ISREG(d->m_fileMode))
+        return false;
+
+    // only if readable
+    if (!isReadable())
+        return false;
+
+    // return true if desktop file
+    return determineMimeType()->is("application/x-desktop");
+}
+
