@@ -30,6 +30,8 @@ class KDirModelTest : public QObject
     Q_OBJECT
 private Q_SLOTS:
     void initTestCase();
+    void cleanupTestCase();
+    void cleanup();
     void testRowCount();
     void testIndex();
     void testNames();
@@ -37,12 +39,23 @@ private Q_SLOTS:
     void testIndexForItem();
     void testData();
     void testReload();
-    void testCreateFile();
     void testModifyFile();
     void testRenameFile();
+    void testRenameDirectory();
+    void testChmodDirectory();
     void testExpandToUrl_data();
     void testExpandToUrl();
+    void testFilter();
+    void testMimeFilter();
+    void testShowHiddenFiles();
+    void testMultipleSlashes();
+    void testUrlWithRef();
+    void testUrlWithHost();
+    void testZipFile();
+    // These two must be done last
     void testDeleteFile();
+    void testDeleteFiles();
+    void testRenameFileToHidden();
 
 protected Q_SLOTS: // 'more private than private slots' - i.e. not seen by qtestlib
     void slotListingCompleted();
@@ -50,15 +63,17 @@ protected Q_SLOTS: // 'more private than private slots' - i.e. not seen by qtest
     void slotRowsInserted(const QModelIndex& index, int, int);
 
 private:
+    void recreateTestData();
     void enterLoop();
-    void fillModel( bool reload );
+    void fillModel(bool reload, bool expectAllIndexes = true);
+    void collectKnownIndexes();
 
 private:
     QEventLoop m_eventLoop;
-    KTempDir m_tempDir;
+    KTempDir* m_tempDir;
     KDirModel m_dirModel;
     QModelIndex m_fileIndex;
-    QModelIndex m_slashFileIndex;
+    QModelIndex m_specialFileIndex;
     QModelIndex m_secondFileIndex;
     QModelIndex m_dirIndex;
     QModelIndex m_fileInDirIndex;
@@ -66,7 +81,9 @@ private:
     QStringList m_topLevelFileNames; // files only
 
     // for slotExpand
-    QString m_nextExpectedExpandPath;
+    QStringList m_expectedExpandSignals;
+    int m_nextExpectedExpandSignals; // index into m_expectedExpandSignals
+    KDirModel* m_dirModelForExpand;
     bool m_rowsInsertedEmitted;
 };
 
